import sys

from cqu_kb.config.config import config, Config
from cqu_kb.utils import check_user, log, check_output_path
from cqu_kb.version import __version__
from cqu_kb.core import get_cal, get_payload
from cqu_jxgl import Student


def server_main(username, password, path):
    main(username, password, path)


def local_main():
    username, password = check_user()
    check_output_path()
    main(username, password, config["output"]["path"])


def main(username, password, path):
    student = Student(
        username=username,
        password=password
    )

    student.login()

    cal = get_cal(
        student.post(
            url="/znpk/Pri_StuSel_rpt.aspx",
            headers={"Content-Type": "application/x-www-form-urlencoded;charset=utf-8"},
            data=get_payload(student)
        ).content
    )

    with open(path, 'wb') as f:
        f.write(cal.to_ical())

    log(f'课表已经保存到{path}')


def console_main():
    import argparse

    def parse_args() -> argparse.Namespace:
        """Parse the command line arguments for the `cqu jwc` binary.

        :return: Namespace with parsed arguments.
        """
        parser = argparse.ArgumentParser(prog="kb", description="第三方 重庆大学 课表导出工具", )

        parser.add_argument(
            "-v",
            "--version",
            action="version",
            version=f"CQU_kb {__version__}",
            help="显示版本号",
        )
        parser.add_argument(
            "-c",
            "--config_path",
            help="查询配置文件路径",
            action="store_true",
        )
        parser.add_argument(
            "-r", "--reset", help="重置配置项", action="store_true",
        )
        parser.add_argument(
            "-u",
            "--username",
            help="学号",
            type=int,
            default=config["user_info"]["username"],
        )
        parser.add_argument(
            "-p",
            "--password",
            help="密码",
            type=str,
            default=config["user_info"]["password"],
        )
        parser.add_argument(
            "-o",
            "--output",
            help="课表输出路径",
            type=str,
            default=config['output']['path'],
        )
        return parser.parse_args()

    args = parse_args()
    if args.reset:
        Config.reset()
        log("已重置配置文件")
    if args.config_path:
        log(f"配置文件位于 {Config.path}\n")
        sys.exit()

    config.dump()

    local_main()


if __name__ == '__main__':
    local_main()



from pathlib import Path

import yaml


class Config:
    path = Path(__file__).parent / "config.yaml"
    default_path = Path(__file__).parent / "config_default.yaml"

    def __init__(self):
        if not self.path.is_file():
            if not self.default_path.is_file():
                raise FileNotFoundError("所有配置文件都已损坏!")
            else:
                Config.reset()
        self.data = self.read_yaml(self.path)

    @staticmethod
    def read_yaml(file_path):
        data = yaml.load(file_path.read_text(), Loader=yaml.SafeLoader)
        return data

    def __getitem__(self, item):
        return self.data.__getitem__(item)

    def __setitem__(self, key, value):
        self.data.__setitem__(key, value)

    def dump(self):
        self.path.write_text(yaml.dump(self.data, Dumper=yaml.SafeDumper))

    @classmethod
    def reset(cls):
        cls.path.write_text(cls.default_path.read_text())


config = Config()



import re
import sys
import time
import traceback
from datetime import datetime
from pathlib import Path

import requests

from cqu_kb.config.config import config
from cqu_kb.version import __version__

ERROR_COUNT = 0


def check_output_path():
    if config['output']['path'] is None:
        flag = False
        for i in ["Desktop", "桌面", "desktop"]:
            if (Path.home() / i).is_dir():
                flag = True
                break
        if flag:
            config['output']['path'] = Path.home() / i / "课表.ics"
        else:
            config['output']['path'] = Path("./课表.ics").absolute()


def exit():
    print("[{}]  遭遇不可抗的错误，程序完全退出".format(datetime.now()))
    sys.exit(1)


def log(msg, error=False, warning=False):
    global ERROR_COUNT
    if error:
        print("[{}]  {} - {}".format(datetime.now(), msg, ERROR_COUNT))
        if config["behavior"]["print_traceback"]:
            traceback.print_exc()
        time.sleep(min(2 ** ERROR_COUNT, config["behavior"]["exp_backoff_limit"]))
        ERROR_COUNT += 1
    elif warning:
        print("[{}]  {}".format(datetime.now(), msg))
    else:
        print("[{}]  {}".format(datetime.now(), msg))
        reset_error_count()


def reset_error_count():
    global ERROR_COUNT
    ERROR_COUNT = 0


def check_user():
    if (
            config["user_info"]["username"] is None
            or config["user_info"]["password"] is None
    ):
        print("未找到有效的帐号和密码，请输入你的帐号和密码，它们将被保存在你的电脑上以备下次使用")
        try:
            config["user_info"]["username"] = input("帐号>>>")
            config["user_info"]["password"] = input("密码>>>")
        except (KeyboardInterrupt, EOFError):
            log("需要输入你的帐号和密码，它们将被保存在你的电脑上以备下次使用")
            exit()
        config.dump()
    return config["user_info"]["username"], config["user_info"]["password"]


def check_update(project_name):
    content = requests.get(f"https://pypi.org/project/{project_name}/").content.decode()
    latest_version = re.findall(project_name + r" \d{1,2}\.\d{1,2}\.\d{1,2}", content)[
        0
    ].lstrip(project_name + " ")
    if latest_version.split(".") > __version__.split("."):
        log(
            f"{project_name}的最新版本为{latest_version}，当前安装的是{__version__}，建议使用`pip install {project_name} -U`来升级",
            warning=True,
        )



from pathlib import Path

from setuptools import setup, find_packages

version = "0.2.1"

# Read the contents of README file
source_root = Path(".")
with (source_root / "README.md").open(encoding="utf-8") as f:
    long_description = f.read()

# Read the requirements
with (source_root / "requirements.txt").open(encoding="utf8") as f:
    requirements = f.readlines()

with (source_root / "src" / "cqu_kb" / "version.py").open(
        "w", encoding="utf-8"
) as f:
    f.writelines(
        [
            '"""This file is auto-generated by setup.py, please do not alter."""\n',
            f'__version__ = "{version}"\n',
            "",
        ]
    )

setup(
    name="cqu-kb",
    version=version,
    description="一个导出重庆大学课程表的第三方工具",
    author="CQU-AI",
    author_email="peter@mail.loopy.tech",
    url="https://github.com/CQU-AI/cqu-kb",
    license="GPL License",
    packages=find_packages("src"),
    package_dir={"": "src"},
    package_data={"": ["config_default.yaml"]},
    platforms=["all"],
    install_requires=requirements,
    python_requires=">=3.6",
    long_description=long_description,
    long_description_content_type="text/markdown",
    entry_points={"console_scripts": ["cqu-kb=cqu_kb:console_main", ]},
)



from datetime import datetime, timedelta
from urllib import parse

import pytz
from bs4 import BeautifulSoup as BS
from icalendar import Calendar, Event

from cqu_kb.config import config


def get_payload(student):
    """
    This function prepares the payload to be sent in HTTP POST method. There are three main fields in the payload. The
    only thing that should be paid attention to is "Sel_XNXQ" field, which denotes the years（学年） and terms（学期）.

    :param student: an instance of Student from cqu-jxgl
    :return: payload to be used in HTTP POST
    """
    # First Get. Won't get any information but a cookie.
    response = student.get("/znpk/Pri_StuSel.aspx")
    soup = BS(response.content, features="html.parser")
    return parse.urlencode({
        "Sel_XNXQ": soup.find("option")['value'],
        "rad": "on",
        "px": "1",
    })


def get_cal(page_content):
    soup = BS(page_content, features="html.parser")
    tables = soup.find_all(class_="page_table")

    courses_info = []
    for i in range(len(tables) // 2):
        table = tables[i * 2 + 1]
        rows = table.find('tbody').find_all('tr')
        cols_num = len(rows[0].find_all('td'))
        for row in rows:
            course_info = {}
            cols = row.find_all('td')
            course_info.update({'course_name': cols[1].text.split(']')[-1] + ("" if cols_num == 13 else "（实验）")})
            course_info.update({'teacher': cols[
                -4 if cols_num == 13 else -5
            ].text})
            course_info.update({'weeks': cols[-3].text})
            course_info.update({'time': cols[-2].text.replace('节]', '')})
            course_info.update({'location': cols[-1].text})
            courses_info.append(course_info)
    for i in range(len(courses_info)):
        if courses_info[i]['course_name'] == '' or courses_info[i]['course_name'] == '（实验）':
            courses_info[i]['course_name'] = courses_info[i - 1]['course_name']

    chinese_to_numbers = {
        '一': 0,
        '二': 1,
        '三': 2,
        '四': 3,
        '五': 4,
        '六': 5,
        '日': 6
    }

    coursenum_to_time = {
        '1': (timedelta(hours=8, minutes=30), timedelta(hours=9, minutes=15)),
        '2': (timedelta(hours=9, minutes=25), timedelta(hours=10, minutes=10)),
        '3': (timedelta(hours=10, minutes=30), timedelta(hours=11, minutes=15)),
        '4': (timedelta(hours=11, minutes=25), timedelta(hours=12, minutes=10)),
        '5': (timedelta(hours=13, minutes=30), timedelta(hours=14, minutes=15)),
        '6': (timedelta(hours=14, minutes=25), timedelta(hours=15, minutes=10)),
        '7': (timedelta(hours=15, minutes=20), timedelta(hours=16, minutes=5)),
        '8': (timedelta(hours=16, minutes=25), timedelta(hours=17, minutes=10)),
        '9': (timedelta(hours=17, minutes=20), timedelta(hours=18, minutes=5)),
        '10': (timedelta(hours=19, minutes=0), timedelta(hours=19, minutes=45)),
        '11': (timedelta(hours=19, minutes=55), timedelta(hours=20, minutes=40)),
        '12': (timedelta(hours=20, minutes=50), timedelta(hours=21, minutes=35)),
        '13': (timedelta(hours=21, minutes=45), timedelta(hours=22, minutes=30)),
    }

    term_start_time = datetime(
        year=int(config["term_start_time"]["year"]),
        month=int(config["term_start_time"]["month"]),
        day=int(config["term_start_time"]["day"]),
        tzinfo=pytz.timezone('Asia/Shanghai')
    )

    events = []

    for course in courses_info:
        week_nums = []
        week_segs = course['weeks'].split(',')
        for seg in week_segs:
            delimiter = [int(i) for i in seg.split('-')]
            start = delimiter[0]
            end = delimiter[1] if len(delimiter) == 2 else start
            for i in range(start, end + 1):
                week_nums.append(i)
        day = chinese_to_numbers[course['time'].split('[')[0]]
        seg = course['time'].split('[')[1].split('-')
        if len(seg) == 2:
            inweek_delta_start = timedelta(days=day) + coursenum_to_time[seg[0]][0]
            inweek_delta_end = timedelta(days=day) + coursenum_to_time[seg[1]][1]
            for week_num in week_nums:
                event_start_datetime = term_start_time + (week_num - 1) * timedelta(days=7) + inweek_delta_start
                event_end_datetime = term_start_time + (week_num - 1) * timedelta(days=7) + inweek_delta_end
                event = Event()
                event.add('summary', '[{}]{}'.format(course['teacher'], course['course_name']))
                event.add('location', course['location'])
                event.add('dtstart', event_start_datetime)
                event.add('dtend', event_end_datetime)
                events.append(event)

    cal = Calendar()
    cal.add('prodid', f'-//重庆大学课表//{config["user_info"]["username"]}//Powered By cqu-kb//')
    cal.add('version', '2.0')
    for event in events:
        cal.add_component(event)
    return cal
